@@ -32,11 +32,11 @@
 #include <fcntl.h>
 #include <string.h>
 
+#include <glib.h>
 #include <glib/gstdio.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <gio/gunixmounts.h>
-#include <glib/gurifuncs.h>
 
 #include "gvfsbackendcomputer.h"
 #include "gvfsmonitor.h"
@@ -310,6 +310,9 @@ recompute_files (GVfsBackendComputer *backend)
   char *basename, *filename;
   const char *extension;
   int uniq;
+  gchar *s;
+  gchar *display_name;
+  gchar *drive_name;
 
   volume_monitor = backend->volume_monitor;
 
@@ -382,7 +385,7 @@ recompute_files (GVfsBackendComputer *backend)
     {
       mount = l->data;
       volume = g_mount_get_volume (mount);
-      if (volume == NULL)
+      if (volume == NULL && !g_mount_is_shadowed (mount))
         {
           file = g_slice_new0 (ComputerFile);
           file->drive = NULL;
@@ -407,16 +410,40 @@ recompute_files (GVfsBackendComputer *backend)
 
       if (file->mount)
         {
+          if (file->drive != NULL)
+            {
+              drive_name = g_drive_get_name (file->drive);
+              s = g_mount_get_name (file->mount);
+              display_name = g_strdup_printf ("%s: %s", drive_name, s);
+              g_free (s);
+              g_free (drive_name);
+            }
+          else
+            {
+              display_name = g_mount_get_name (file->mount);
+            }
           file->icon = g_mount_get_icon (file->mount);
-          file->display_name = g_mount_get_name (file->mount);
+          file->display_name = display_name;
           file->root = g_mount_get_root (file->mount);
           file->can_unmount = g_mount_can_unmount (file->mount);
           file->can_eject = g_mount_can_eject (file->mount);
         }
       else if (file->volume)
         {
+          if (file->drive != NULL)
+            {
+              drive_name = g_drive_get_name (file->drive);
+              s = g_volume_get_name (file->volume);
+              display_name = g_strdup_printf ("%s: %s", drive_name, s);
+              g_free (s);
+              g_free (drive_name);
+            }
+          else
+            {
+              display_name = g_volume_get_name (file->volume);
+            }
           file->icon = g_volume_get_icon (file->volume);
-          file->display_name = g_volume_get_name (file->volume);
+          file->display_name = display_name;
           file->can_mount = g_volume_can_mount (file->volume);
           file->root = NULL;
           file->can_eject = g_volume_can_eject (file->volume);
@@ -726,10 +753,9 @@ try_create_dir_monitor (GVfsBackend *backend,
 
   if (file != &root)
     {
-      if (file != NULL)
-        g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
-                          G_IO_ERROR_NOT_SUPPORTED,
-                          _("Can't open mountable file"));
+      g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
+			G_IO_ERROR_NOT_SUPPORTED,
+			_("Not supported"));
       
       return TRUE;
     }
