@@ -505,7 +505,7 @@ ftp_connection_receive (FtpConnection *conn,
  * codes other than 2xx cause an error. The last read ftp command will be put 
  * into @conn->read_buffer.
  *
- * Returns: 0 on error or the receied FTP code otherwise.
+ * Returns: 0 on error or the received FTP code otherwise.
  *     
  **/
 static guint
@@ -787,10 +787,6 @@ ftp_connection_prepare (FtpConnection *conn)
     conn->workarounds |= FTP_WORKAROUND_FEAT_AFTER_LOGIN;
     conn->features = 0;
   }
-
-  /* instruct server that we'll give and assume we get utf8 */
-  if (conn->features & FTP_FEATURE_UTF8)
-    ftp_connection_send (conn, 0, "OPTS UTF8 ON");
 }
 
 static gboolean
@@ -822,14 +818,21 @@ ftp_connection_use (FtpConnection *conn)
     if (ftp_connection_send (conn, 0, "FEAT") != 0) {
       ftp_connection_parse_features (conn);
     } else {
-      ftp_connection_clear_error(conn);
-      conn->features = FTP_FEATURE_EPSV;
+      ftp_connection_clear_error (conn);
+      conn->features = FTP_FEATURES_DEFAULT;
     }
   }
 
+  /* instruct server that we'll give and assume we get utf8 */
+  if (conn->features & FTP_FEATURE_UTF8) 
+    {
+      if (!ftp_connection_send (conn, 0, "OPTS UTF8 ON"))
+        ftp_connection_clear_error (conn);
+    }
+
   if (ftp_connection_send (conn, 0, "SYST"))
     ftp_connection_parse_system (conn);
-  ftp_connection_clear_error(conn);
+  ftp_connection_clear_error (conn);
 
   return TRUE;
 }
@@ -1236,7 +1239,7 @@ dir_default_iter_process (gpointer        iter,
   else if (symlink)
     *symlink = NULL;
 
-  g_file_info_set_size (info, strtoul (result.fe_size, NULL, 10));
+  g_file_info_set_size (info, g_ascii_strtoull (result.fe_size, NULL, 10));
 
   gvfs_file_info_populate_default (info, s,
 				   type == 'f' ? G_FILE_TYPE_REGULAR :
@@ -2228,7 +2231,7 @@ create_file_info (GVfsBackendFtp *ftp, FtpConnection *conn, const char *filename
 
           gvfs_file_info_populate_default (info, filename, G_FILE_TYPE_REGULAR);
 
-          g_file_info_set_size (info, strtoul (conn->read_buffer+4, NULL, 0));
+          g_file_info_set_size (info, g_ascii_strtoull (conn->read_buffer+4, NULL, 0));
 
           g_file_info_set_is_hidden (info, TRUE);
         }
