@@ -33,16 +33,18 @@
 #include "gmountoperationdbus.h"
 #include "gvfsdaemonprotocol.h"
 #include "gdbusutils.h"
-#include <glib/gurifuncs.h>
+#include <glib.h>
 #include <gio/gio.h>
 
 typedef struct {
   char *display_name;
   char *stable_name;
+  char *x_content_types;
   char *icon;
   char *prefered_filename_encoding;
   gboolean user_visible;
-
+  char *fuse_mountpoint; /* Always set, even if fuse not availible */
+  
   /* Daemon object ref */
   char *dbus_id;
   char *object_path;
@@ -56,6 +58,10 @@ typedef struct  {
   char *exec;
   char *dbus_name;
   gboolean automount;
+  char *scheme;
+  char **scheme_aliases;
+  int default_port;
+  gboolean hostname_is_inet;
 } VfsMountable; 
 
 typedef void (*MountCallback) (VfsMountable *mountable,
@@ -94,6 +100,32 @@ find_vfs_mount (const char *dbus_id,
   return NULL;
 }
 
+static VfsMount *
+find_vfs_mount_by_fuse_path (const char *fuse_path)
+{
+  GList *l;
+
+  if (!fuse_available)
+    return NULL;
+  
+  for (l = mounts; l != NULL; l = l->next)
+    {
+      VfsMount *mount = l->data;
+
+      if (mount->fuse_mountpoint != NULL &&
+	  g_str_has_prefix (fuse_path, mount->fuse_mountpoint))
+	{
+	  int len = strlen (mount->fuse_mountpoint);
+	  if (fuse_path[len] == 0 ||
+	      fuse_path[len] == '/')
+	    return mount;
+	}
+    }
+  
+  return NULL;
+}
+
+
 static VfsMount *
 match_vfs_mount (GMountSpec *match)
 {
@@ -143,6 +175,8 @@ vfs_mountable_free (VfsMountable *mountable)
   g_free (mountable->type);
   g_free (mountable->exec);
   g_free (mountable->dbus_name);
+  g_free (mountable->scheme);
+  g_strfreev (mountable->scheme_aliases);
   g_free (mountable);
 }
 
@@ -151,7 +185,9 @@ vfs_mount_free (VfsMount *mount)
 {
   g_free (mount->display_name);
   g_free (mount->stable_name);
+  g_free (mount->x_content_types);
   g_free (mount->icon);
+  g_free (mount->fuse_mountpoint);
   g_free (mount->prefered_filename_encoding);
   g_free (mount->dbus_id);
   g_free (mount->object_path);
@@ -193,6 +229,11 @@ vfs_mount_to_dbus (VfsMount *mount,
 				       DBUS_TYPE_STRING,
 				       &mount->stable_name))
     _g_dbus_oom ();
+
+  if (!dbus_message_iter_append_basic (&struct_iter,
+				       DBUS_TYPE_STRING,
+				       &mount->x_content_types))
+    _g_dbus_oom ();
   
   if (!dbus_message_iter_append_basic (&struct_iter,
 				       DBUS_TYPE_STRING,
@@ -210,21 +251,10 @@ vfs_mount_to_dbus (VfsMount *mount,
 				       &user_visible))
     _g_dbus_oom ();
 	      
-  
-  fuse_mountpoint = NULL;
-  if (fuse_available && mount->user_visible)
-    {
-      char *fs_name;
-      
-      /* Keep in sync with fuse daemon */
-      fs_name = g_uri_escape_string (mount->stable_name, "+@#$., ", TRUE);
-      
-      fuse_mountpoint = g_build_filename (g_get_home_dir(), ".gvfs", fs_name, NULL);
-    }
-  
-  if (fuse_mountpoint == NULL)
-    fuse_mountpoint = g_strdup ("");
 
+  fuse_mountpoint = "";
+  if (fuse_available && mount->fuse_mountpoint)
+    fuse_mountpoint = mount->fuse_mountpoint;
   _g_dbus_message_iter_append_cstring (&struct_iter, fuse_mountpoint);
 
   g_mount_spec_to_dbus (&struct_iter, mount->mount_spec);
@@ -233,6 +263,60 @@ vfs_mount_to_dbus (VfsMount *mount,
     _g_dbus_oom ();
 }
 
+static void
+vfs_mountable_to_dbus (VfsMountable *mountable,
+		       DBusMessageIter *iter)
+{
+  DBusMessageIter struct_iter;
+  dbus_bool_t bool;
+  guint32 int32;
+  char *s;
+  char **a;
+  char *empty[] = {NULL};
+  
+  if (!dbus_message_iter_open_container (iter,
+					 DBUS_TYPE_STRUCT,
+					 NULL,
+					 &struct_iter))
+    _g_dbus_oom ();
+
+  if (!dbus_message_iter_append_basic (&struct_iter,
+				       DBUS_TYPE_STRING,
+				       &mountable->type))
+    _g_dbus_oom ();
+  
+  s = mountable->scheme;
+  if (s == NULL)
+    s = "";
+  if (!dbus_message_iter_append_basic (&struct_iter,
+				       DBUS_TYPE_STRING,
+				       &s))
+    _g_dbus_oom ();
+
+  a = mountable->scheme_aliases;
+  if (a == NULL)
+    a = empty;
+  _g_dbus_message_iter_append_args (&struct_iter,
+				    DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &a, (int)g_strv_length (a),
+				    0);
+  
+  int32 = mountable->default_port;
+  if (!dbus_message_iter_append_basic (&struct_iter,
+				       DBUS_TYPE_INT32,
+				       &int32))
+    _g_dbus_oom ();
+  
+  bool = mountable->hostname_is_inet;
+  if (!dbus_message_iter_append_basic (&struct_iter,
+				       DBUS_TYPE_BOOLEAN,
+				       &bool))
+    _g_dbus_oom ();
+
+  if (!dbus_message_iter_close_container (iter, &struct_iter))
+    _g_dbus_oom ();
+}
+
+
 /************************************************************************
  * Support for mounting a VfsMountable                                  *
  ************************************************************************/
@@ -368,14 +452,14 @@ spawn_mount_message_function (DBusConnection  *connection,
 				  DBUS_TYPE_STRING, &error_message,
 				  DBUS_TYPE_INVALID))
 	{
-	  g_set_error (&error, G_IO_ERROR, G_IO_ERROR_FAILED,
-		       _("Invalid arguments from spawned child"));
+	  g_set_error_literal (&error, G_IO_ERROR, G_IO_ERROR_FAILED,
+			       _("Invalid arguments from spawned child"));
 	  mount_finish (data, error);
 	  g_error_free (error);
 	}
       else if (!succeeded)
 	{
-	  g_set_error (&error, G_IO_ERROR, G_IO_ERROR_FAILED, error_message);
+	  g_set_error_literal (&error, G_IO_ERROR, G_IO_ERROR_FAILED, error_message);
 	  mount_finish (data, error);
 	  g_error_free (error);
 	}
@@ -405,8 +489,8 @@ spawn_mount (MountData *data)
   error = NULL;
   if (data->mountable->exec == NULL)
     {
-      g_set_error (&error, G_IO_ERROR, G_IO_ERROR_FAILED,
-		   "No exec key defined for mountpoint");
+      g_set_error_literal (&error, G_IO_ERROR, G_IO_ERROR_FAILED,
+			   "No exec key defined for mountpoint");
       mount_finish (data, error);
       g_error_free (error);
     }
@@ -496,6 +580,14 @@ read_mountable_config (void)
 			  mountable->exec = g_key_file_get_string (keyfile, "Mount", "Exec", NULL);
 			  mountable->dbus_name = g_key_file_get_string (keyfile, "Mount", "DBusName", NULL);
 			  mountable->automount = g_key_file_get_boolean (keyfile, "Mount", "AutoMount", NULL);
+			  mountable->scheme = g_key_file_get_string (keyfile, "Mount", "Scheme", NULL);
+			  mountable->scheme_aliases =
+			    g_key_file_get_string_list (keyfile, "Mount", "SchemeAliases", NULL, NULL);
+			  mountable->default_port = g_key_file_get_integer (keyfile, "Mount", "DefaultPort", NULL);
+			  mountable->hostname_is_inet = g_key_file_get_boolean (keyfile, "Mount", "HostnameIsInetAddress", NULL);
+
+			  if (mountable->scheme == NULL)
+			    mountable->scheme = g_strdup (mountable->type);
 			  
 			  mountables = g_list_prepend (mountables, mountable);
 			}
@@ -506,6 +598,7 @@ read_mountable_config (void)
 	  g_key_file_free (keyfile);
 	  g_free (path);
 	}
+      g_dir_close (dir);
     }
 }
 
@@ -578,7 +671,7 @@ register_mount (DBusConnection *connection,
   VfsMount *mount;
   DBusMessage *reply;
   DBusError error;
-  const char *display_name, *stable_name, *icon, *obj_path, *id, *prefered_filename_encoding;
+  const char *display_name, *stable_name, *x_content_types, *icon, *obj_path, *id, *prefered_filename_encoding;
   dbus_bool_t user_visible;
   DBusMessageIter iter;
   GMountSpec *mount_spec;
@@ -593,6 +686,7 @@ register_mount (DBusConnection *connection,
 				     DBUS_TYPE_OBJECT_PATH, &obj_path,
 				     DBUS_TYPE_STRING, &display_name,
                                      DBUS_TYPE_STRING, &stable_name,
+                                     DBUS_TYPE_STRING, &x_content_types,
 				     DBUS_TYPE_STRING, &icon,
 				     DBUS_TYPE_STRING, &prefered_filename_encoding,
 				     DBUS_TYPE_BOOLEAN, &user_visible,
@@ -615,12 +709,23 @@ register_mount (DBusConnection *connection,
 	  mount = g_new0 (VfsMount, 1);
 	  mount->display_name = g_strdup (display_name);
           mount->stable_name = g_strdup (stable_name);
+          mount->x_content_types = g_strdup (x_content_types);
 	  mount->icon = g_strdup (icon);
 	  mount->prefered_filename_encoding = g_strdup (prefered_filename_encoding);
 	  mount->user_visible = user_visible;
 	  mount->dbus_id = g_strdup (id);
 	  mount->object_path = g_strdup (obj_path);
 	  mount->mount_spec = mount_spec;
+
+	  if (user_visible)
+	    {
+	      char *fs_name;
+	      
+	      /* Keep in sync with fuse daemon */
+	      fs_name = g_uri_escape_string (mount->stable_name, "+@#$., ", TRUE);
+	      
+	      mount->fuse_mountpoint = g_build_filename (g_get_home_dir(), ".gvfs", fs_name, NULL);
+	    }
 	  
 	  mounts = g_list_prepend (mounts, mount);
 
@@ -689,7 +794,7 @@ maybe_automount (GMountSpec *spec,
       AutoMountData *data;
       GMountSource *mount_source;
 
-      g_print ("automounting...\n");
+      g_debug ("automounting...\n");
 
       mount_source = g_mount_source_new_dummy ();
 
@@ -756,6 +861,48 @@ lookup_mount (DBusConnection *connection,
     dbus_connection_send (connection, reply, NULL);
 }
 
+static void
+lookup_mount_by_fuse_path (DBusConnection *connection,
+			   DBusMessage *message)
+{
+  VfsMount *mount;
+  DBusMessage *reply;
+  DBusMessageIter iter;
+  char *fuse_path;
+
+  dbus_message_iter_init (message, &iter);
+
+  reply = NULL;
+  if (_g_dbus_message_iter_get_args (&iter, NULL, 
+				     G_DBUS_TYPE_CSTRING, &fuse_path,
+				     0))
+    {
+      mount = find_vfs_mount_by_fuse_path (fuse_path);
+
+      if (mount == NULL)
+	reply = _dbus_message_new_gerror (message,
+					  G_IO_ERROR,
+					  G_IO_ERROR_NOT_MOUNTED,
+					  _("The specified location is not mounted"));
+      else
+	{
+	  reply = dbus_message_new_method_return (message);
+	  if (reply)
+	    {
+	      dbus_message_iter_init_append (reply, &iter);
+	      vfs_mount_to_dbus (mount, &iter);
+	    }
+	}
+    }
+  else
+    reply = dbus_message_new_error (message,
+				    DBUS_ERROR_INVALID_ARGS,
+				    "Invalid arguments");
+  
+  if (reply != NULL)
+    dbus_connection_send (connection, reply, NULL);
+}
+
 static void
 list_mounts (DBusConnection *connection,
 	     DBusMessage *message)
@@ -781,6 +928,7 @@ list_mounts (DBusConnection *connection,
 					   DBUS_TYPE_STRING_AS_STRING
 					   DBUS_TYPE_STRING_AS_STRING
 					   DBUS_TYPE_STRING_AS_STRING
+					   DBUS_TYPE_STRING_AS_STRING
 					   DBUS_TYPE_BOOLEAN_AS_STRING
 					   DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_BYTE_AS_STRING
  					   G_MOUNT_SPEC_TYPE_AS_STRING
@@ -942,6 +1090,47 @@ list_mount_types (DBusConnection *connection,
   dbus_connection_send (connection, reply, NULL);
 }
 
+static void
+list_mountable_info (DBusConnection *connection,
+		     DBusMessage *message)
+{
+  VfsMountable *mountable;
+  DBusMessage *reply;
+  DBusMessageIter iter, array_iter;
+  GList *l;
+
+  reply = dbus_message_new_method_return (message);
+  if (reply == NULL)
+    _g_dbus_oom ();
+
+  dbus_message_iter_init_append (reply, &iter);
+
+  
+  if (!dbus_message_iter_open_container (&iter,
+					 DBUS_TYPE_ARRAY,
+					 DBUS_STRUCT_BEGIN_CHAR_AS_STRING
+					   DBUS_TYPE_STRING_AS_STRING /* type */
+					   DBUS_TYPE_STRING_AS_STRING /* scheme */
+					   DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_STRING_AS_STRING /* scheme aliases */
+					   DBUS_TYPE_INT32_AS_STRING /* default port */
+					   DBUS_TYPE_BOOLEAN_AS_STRING /* host is inet */
+					 DBUS_STRUCT_END_CHAR_AS_STRING,
+					 &array_iter))
+    _g_dbus_oom ();
+
+  for (l = mountables; l != NULL; l = l->next)
+    {
+      mountable = l->data;
+      
+      vfs_mountable_to_dbus (mountable, &array_iter);
+    }
+
+  if (!dbus_message_iter_close_container (&iter, &array_iter))
+    _g_dbus_oom ();
+  
+  dbus_connection_send (connection, reply, NULL);
+}
+
 static DBusHandlerResult
 dbus_message_function (DBusConnection  *connection,
 		       DBusMessage     *message,
@@ -962,6 +1151,10 @@ dbus_message_function (DBusConnection  *connection,
 					G_VFS_DBUS_MOUNTTRACKER_INTERFACE,
 					G_VFS_DBUS_MOUNTTRACKER_OP_LOOKUP_MOUNT))
     lookup_mount (connection, message, TRUE);
+  else if (dbus_message_is_method_call (message,
+					G_VFS_DBUS_MOUNTTRACKER_INTERFACE,
+					G_VFS_DBUS_MOUNTTRACKER_OP_LOOKUP_MOUNT_BY_FUSE_PATH))
+    lookup_mount_by_fuse_path (connection, message);
   else if (dbus_message_is_method_call (message,
 					G_VFS_DBUS_MOUNTTRACKER_INTERFACE,
 					G_VFS_DBUS_MOUNTTRACKER_OP_LIST_MOUNTS))
@@ -974,6 +1167,10 @@ dbus_message_function (DBusConnection  *connection,
   					G_VFS_DBUS_MOUNTTRACKER_INTERFACE,
 					G_VFS_DBUS_MOUNTTRACKER_OP_LIST_MOUNT_TYPES))
     list_mount_types (connection, message);
+  else if (dbus_message_is_method_call (message,
+  					G_VFS_DBUS_MOUNTTRACKER_INTERFACE,
+					G_VFS_DBUS_MOUNTTRACKER_OP_LIST_MOUNTABLE_INFO))
+    list_mountable_info (connection, message);
   else
     res = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
   
